#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <linux/if_ether.h>
#include <net/if_arp.h>
#include <linux/if_packet.h>
#include <string.h>

#define PCAP_ERRBUF_SIZE 64

/*
 * Not all systems have strerror().
 */
char *pcap_strerror(int errnum)
{
  return strerror(errnum);
}


/*
 *  Return the index of the given device name. Fill ebuf and return 
 *  -1 on failure.
 */
static int
iface_get_id(int fd, const char *device, char *ebuf)
{
	struct ifreq	ifr;

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));

	if (ioctl(fd, SIOCGIFINDEX, &ifr) == -1) {
		snprintf(ebuf, PCAP_ERRBUF_SIZE,
			 "ioctl: %s", pcap_strerror(errno));
		return -1;
	}

	return ifr.ifr_ifindex;
}

/*
 *  Bind the socket associated with FD to the given device. 
 */
static int
iface_bind(int fd, int ifindex, char *ebuf)
{
	struct sockaddr_ll	sll;

	memset(&sll, 0, sizeof(sll));
	sll.sll_family		= AF_PACKET;
	sll.sll_ifindex		= ifindex;
	sll.sll_protocol	= htons(ETH_P_ALL);

	if (bind(fd, (struct sockaddr *) &sll, sizeof(sll)) == -1) {
		snprintf(ebuf, PCAP_ERRBUF_SIZE,
			 "bind: %s", pcap_strerror(errno));
		return -1;
	}

	return 0;
}

int main(int argc, char **argv)
{
  char ebuf[PCAP_ERRBUF_SIZE];
#define BUFSIZE 40
  unsigned char buf[BUFSIZE];
  int device_id, sock_fd, c;
  
  sock_fd = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
  if(sock_fd == -1) {
    fprintf(stderr, "error opening raw socket\n");
  }
  
  device_id = iface_get_id(sock_fd, "wifi0", ebuf);
  if (device_id == -1) {
    fprintf(stderr, "%s\nplease run as root\n", ebuf);
    return -1;
  }
  
  if (iface_bind(sock_fd, device_id, ebuf) == -1) {
    fprintf(stderr, "%s\n", ebuf);
    return -1;
  }

  memset(buf, 0, BUFSIZE);
  while(read(sock_fd, buf, BUFSIZE)) {
    for(c = 0; c < BUFSIZE/2; c += 2) 
      fprintf(stdout, "%04x%04x ", buf[c], buf[c+1]);
    fprintf(stdout, "\n");
    memset(buf, 0, BUFSIZE);
  }

  close(sock_fd);

  return 0;
}
